<?php
/**
 * Plugin Name: Ajax Product Listing
 * Plugin URI: https://redpishi.com
 * Description: A plugin to display WooCommerce products with AJAX-powered pagination and sorting.
 * Version: 1.8
 * Author: Maya
 * Author URI: https://redpishi.com
 */

// Exit if accessed directly
if ( ! defined( 'ABSPATH' ) ) {
    exit;
}

add_action('template_redirect', function () {
    if (isset($_GET['post_type']) && $_GET['post_type'] === 'product') {
        $query_args = $_GET;
        unset($query_args['post_type']);
        $new_url = home_url(add_query_arg($query_args, ''));
        wp_safe_redirect($new_url);
        exit;
    }
});

// Enqueue Scripts and Styles
function apl_enqueue_assets() {
    wp_enqueue_script(
        'apl-ajax-script',
        plugin_dir_url(__FILE__) . 'assets/js/ajax-product-listing.js',
        ['jquery'],
        '1.3.62',
        true
    );

    wp_enqueue_style(
        'apl-ajax-style',
        plugin_dir_url(__FILE__) . 'assets/css/ajax-product-listing.css',
        [],
        '1.3.62'
    );

    wp_localize_script('apl-ajax-script', 'aplAjax', [
        'ajaxUrl' => admin_url('admin-ajax.php'),
    ]);
}
add_action('wp_enqueue_scripts', 'apl_enqueue_assets');

// Shortcode to Display Products
function apl_ajax_product_list_shortcode($atts) {
    static $instance_count = 0; // Incremental ID for multiple instances

    $instance_id = ++$instance_count;

    $atts = shortcode_atts([
        'cats' => '',
        'tags' => '',
        'sort' => 'default',
        'n'	=> '12',
        'p'	=> '',
    ], $atts, 'ajax-list');

    ob_start();
    $sortings = [
    "default" => "Default sorting",
    "popularity" => "Sort by popularity" ,
    "rating" => "Sort by average rating" ,
    "date" => "Sort by latest",
    "price" => "Sort by price: low to high" ,
    "price-desc" => "Sort by price: high to low" ,
    "alphabetical" => "Alphabetical (A-Z)" ,
    "alphabetical-desc" => "Alphabetical (Z-A)" ,
    ];

    $p = esc_attr($atts['p']);
    if ($p == "off") { $none = 'style="display: none;"';} else { $none = ""; }
    $per_page = $n = esc_attr($atts['n']);
    if ( $n == "" ) { $per_page = $n = 12; }
    $sort = esc_attr($atts['sort']);
    $cats = esc_attr($atts['cats']);
    $tags = esc_attr($atts['tags']);
    $s = isset($_GET['s']) ? $_GET['s'] : '';
    if (array_key_exists($sort, $sortings)) {
		$defult_sort_txt = $sortings[$sort];
		unset($sortings[ $sort ]);
	} else { $defult_sort_txt  = ""; $sort = "default"; }

	$product_html_getter = product_list_html( $per_page, 1, $cats, $sort, $tags, $s );

    ?>
    <div id="ajax-product-list-<?php echo esc_attr($instance_id); ?>" class="ajax-product-list" data-instance-id="<?php echo esc_attr($instance_id); ?>">
        <div class="controls" <?= $none ?>>
            <div class="sorting-controls"  >
                <select class="sorting-dropdown">
					<option value="<?= $sort ?>"><?= $defult_sort_txt ?></option>
					<?php
					foreach ( $sortings as $k => $v ) {
						?>
							<option value="<?= $k ?>"><?= $v ?></option>
						<?php
						}
					?>
                </select>
                <div class="toggle-view-wrapper">
                <button class="toggle-view grid-view">
					<svg xmlns="http://www.w3.org/2000/svg" width="16" height="16" viewBox="0 0 24 24" fill="none" stroke="currentColor" stroke-width="2" stroke-linecap="round" stroke-linejoin="round">
					  <line x1="3" y1="6" x2="21" y2="6"></line>
					  <line x1="3" y1="12" x2="21" y2="12"></line>
					  <line x1="3" y1="18" x2="21" y2="18"></line>
					</svg>
                </button>
                <button class="toggle-view list-view active">
					<svg xmlns="http://www.w3.org/2000/svg" width="16" height="16" viewBox="0 0 24 24" fill="none" stroke="currentColor" stroke-width="2" stroke-linecap="round" stroke-linejoin="round">
					  <line x1="3" y1="6" x2="21" y2="6"></line>
					  <line x1="3" y1="12" x2="21" y2="12"></line>
					  <line x1="3" y1="18" x2="21" y2="18"></line>
					</svg>
                </button>
                </div>
            </div>
            <div class="right">
            <div class="pagination-controls">
				<?php
					if ( $n == '12' ) { ?>
						<button data-per-page="12" class="active">12</button>
					<?php } else { ?>
						 <button data-per-page="12" data-n="<?= $n ?>">12</button>
						 <?php }
				?>
                <button data-per-page="24">24</button>
                <button data-per-page="48">48</button>
                <button data-per-page="96">96</button>
                <button data-per-page="-1">All</button>
            </div>

			<div class="black hide-mobile">Show</div>
             <div class="apl-result-count black">
				<?php
				// Add a placeholder for the result count, updated dynamically
				echo sprintf(
					'Showing %d–%d of %d results',
					1, // Start
					12, // End (default per page)
					$product_html_getter[3] // Total products
				);
				?>
			</div>

        </div></div>
        <div class="product-list-container"> <?= $product_html_getter[0] ?> </div>
        <div class="more" <?= $none ?> > <?= $product_html_getter[1] ?> </div>
        <input type="hidden" class="product-list-cats" value="<?php echo esc_attr($atts['cats']); ?>">
        <input type="hidden" class="product-list-tags" value="<?php echo esc_attr($atts['tags']); ?>">
        <input type="hidden" class="product-list-s" value="<?php echo $s; ?>">
        <input type="hidden" class="product-list-sort" value="<?php echo esc_attr($atts['sort']); ?>">
    </div>
    <?php
    return ob_get_clean();
}
add_shortcode('ajax-list', 'apl_ajax_product_list_shortcode');

// AJAX Handler to Load Products

function apl_load_products() {
    $paged = isset($_POST['paged']) ? intval($_POST['paged']) : 1;
    $per_page = isset($_POST['per_page']) ? intval($_POST['per_page']) : 12;
    $cats = isset($_POST['cats']) ? sanitize_text_field($_POST['cats']) : '';
    $tags = isset($_POST['tags']) ? sanitize_text_field($_POST['tags']) : '';
    $s = isset($_POST['s']) ? sanitize_text_field($_POST['s']) : '';
    $sort = isset($_POST['sort']) ? sanitize_text_field($_POST['sort']) : 'default';

	$product_getter = product_list_html( $per_page, $paged, $cats, $sort, $tags , $s );

    $html = $product_getter[0];


    wp_send_json([
    'products' => $products_html,
    'pagination' => $pagination_html,
    'result_count' => $product_getter[2],
    'html' =>$product_getter[0],
    'more' => $product_getter[1],
]);

    wp_die();
}
add_action('wp_ajax_load_products', 'apl_load_products');
add_action('wp_ajax_nopriv_load_products', 'apl_load_products');




// Hide Woo msg on home page etc
function custom_add_to_cart_message($message, $products) {
    if (!is_shop() && !is_product() && !is_cart() && !is_checkout()) {
        return '';
    }
    return $message;
}
add_filter('wc_add_to_cart_message_html', 'custom_add_to_cart_message', 10, 2);


function product_list_html( $per_page, $paged, $cats, $sort, $tags = "", $s = "" ){

		$args = [
				'post_type'      => 'product',
				'posts_per_page' => $per_page,
				'paged'          => $paged,
			];

		if (!empty($cats)) {
			$args['tax_query'] = [
				[
					'taxonomy' => 'product_cat',
					'field'    => 'term_id',
					'terms'    => explode(',', $cats),
				],
			];
		}

		if (!empty($tags)) { // Assuming $tags contains tag IDs or slugs
			$args['tax_query'][] = [
				'taxonomy' => 'product_tag',
				'field'    => 'term_id', // Change to 'slug' if $tags contains slugs
				'terms'    => explode(',', $tags),
			];
		}

		if (!empty($s)) { // Assuming $search_query contains the search term
			$args['s'] = $s;
		}

		// Sorting
		switch ($sort) {
		case 'popularity':
			$args['orderby'] = 'meta_value_num';
			$args['meta_key'] = 'total_sales';
			break;
		case 'rating':
			$args['orderby'] = 'meta_value_num';
			$args['meta_key'] = '_wc_average_rating';
			break;
		case 'date':
			$args['orderby'] = 'date';
			break;
		case 'price':
			$args['orderby'] = 'meta_value_num';
			$args['meta_key'] = '_price';
			$args['order'] = 'ASC';
			break;
		case 'price-desc':
			$args['orderby'] = 'meta_value_num';
			$args['meta_key'] = '_price';
			$args['order'] = 'DESC';
			break;
		case 'alphabetical':
			$args['orderby'] = 'title';
			$args['order'] = 'ASC';
			break;
		case 'alphabetical-desc':
			$args['orderby'] = 'title';
			$args['order'] = 'DESC';
			break;
		default:
			// Use WooCommerce default sorting
			break;
	}

		$query = new WP_Query($args);
		$all = $query->found_posts;
		$to = $query->post_count ? $query->query_vars['paged'] * $query->query_vars['posts_per_page'] : 0;
		if ($to < 0 ) { $to = $all; }
		$result_count = sprintf(
		'Showing %d–%d of %d results',
		1,
		$to,
		$all
	);

		global $woocommerce;
		if (function_exists('WC') && WC()->cart) {
			$card = WC()->cart->get_cart();
	    } else { $card=[]; }
		if ($query->have_posts()) {
			ob_start();
			while ($query->have_posts()) {
				$query->the_post();
				global $product;
				$id = $product->get_id();

				$is_in_cart = false;
				foreach ( $card as $cart_item) {
					if ($cart_item['product_id'] === $id ) {
						$is_in_cart = true;
						break;
					}
				}
				?>
				<div class="entry loop-entry">
					<div class="product-image">
						<a href="<?php the_permalink(); ?>">
							<?php echo woocommerce_get_product_thumbnail(); ?>
						</a>
					</div>
					<div class="product-details">
						<h3 class="product-title">
							<a href="<?php the_permalink(); ?>"><?php the_title(); ?></a>
						</h3>
						<span class="price"><?php echo $product->get_price_html(); ?></span>
						<?php if ($product->get_average_rating()) : ?>
							<span class="woocommerce">
								<?php echo wc_get_rating_html($product->get_average_rating()); ?>
							</span>
						<?php endif; ?>
						<div class="product-actions">
							<?php if ($is_in_cart) { ?>
								<a href="<?php echo esc_url(wc_get_cart_url() ); ?>"
							   class="button add_to_cart_button view-card"
							   data-product_id="<?php echo esc_attr($id ); ?>"
								data-quantity="1">
								 View Your Cart
								 </a>

							<?php }
							else
							{ ?>
								<a href="<?php echo esc_url($product->add_to_cart_url()); ?>"
							   class="button add_to_cart_button ajax_add_to_cart"
							   data-product_id="<?php echo esc_attr($id ); ?>"
								data-quantity="1">

							  <?php esc_html_e('Add to cart', 'woocommerce'); ?>
							</a>
							<?php } ?>

						</div>
					</div>
				</div>
				<?php
			}

			if ( $to < $all   )
        {
		$loadmorebtn =	'<button type="button" class="load-more" >Load more</button>';
		} else { $loadmorebtn = ""; }

			} else {
        $html = '<p>No products found.</p>';
    }
	wp_reset_postdata();
	$html = ob_get_clean();

	return [$html, $loadmorebtn, $result_count, $all];
}
